@@ -74,6 +74,10 @@ account             = r"anzhu@microsoft.com"
 
 cr                  = CosmosReader(scope_exe_path,account,vc_path)
 
+#%% cosmos delete file test
+file_path = "/users/anzhu/scope_query_temp.ss"
+cr.delete_file_from_cosmos(file_path)
+
 #%% run scope script
 output_str = cr.run_scope(scope_script)
 cr.check_job_status(output_str)
@@ -195,6 +199,12 @@ connect_str         = "DefaultEndpointsProtocol=https;AccountName=cedssparkstora
 container           = "andrewzhu"
 abr = AzureBlobReader(blob_conn_str=connect_str,container_name=container)
 
+#%% blob upload test
+blob_file_path      = r'powerapps_data/pa_daily_temp.csv'
+local_csv_path      = r'data/scope_query_temp.csv'
+abr.upload_file_chunks(blob_file_path=blob_file_path,local_file_path=local_csv_path)
+print('done')
+
 #%% download test
 blob_file_path = 'temp_query_data.csv'
 local_file_path = 'test.csv'
@@ -243,3 +253,5 @@ folder = 'anzhu/test'
 r = kr.create_table_from_csv(kusto_table_name,csv_file_path,kusto_folder=folder)
 print(r)
 
+
+#%% cosmos delete file test
